// graphics --- common for Cell and Cube
//.......................................

bool stop   = false;  // quit the program?
bool showit = true;   // display it?

#include <SDL/SDL.h>

#define SX 1024
#define SY 768
SDL_Surface *surface;

// put a pixel
void putpix(int x, int y, int c) {
  if(x<0 || y<0 || x>=SX || y>=SY) return;
  
  unsigned char *dst = (unsigned char*) surface->pixels;
  dst += y*surface->pitch;
  int* ptr = (int*) dst;
  ptr[x] = c;
  }

// convert a vector from internal coordinates to screen coordinates

vec3 ekran(vec3 x) {
  vec3 v;
  v[0] = x[0]*50 + SX/2;
  v[1] = x[1]*50 + SY/2;
  v[2] = x[2]*50;
  return v;
  }

// what color to use for the given point on a line

int col(ld z, int typ) {
  z /= 3;
  z += 128;
  if(z < 0) return 0x800000;
  if(z > 255) return 0x0000FF;
  if(typ == 1) return 0x10101 * int(z);
  if(typ == 2) return 0x10000 * int(z);
  if(typ == 3) return 0x00100 * int(z);
  }

// SVG file
FILE *f;

// add a line to SVG

void addLine(vec3 v1, vec3 v2, const char *w) {
  fprintf(f, "<line x1=\"%.1f\" y1=\"%.1f\" x2=\"%.1f\" y2=\"%.1f\" style=\"stroke:black;stroke-width:%s\"/>\n",
    float(v1[0]), float(v1[1]),
    float(v2[0]), float(v2[1]),
    w);
  }

bool pressed[65536];

// rotate the picture

void roll() {

  ld alfa = .1;
  
  if(pressed[SDLK_HOME] || pressed[SDLK_KP7]) alfa = .01;
  
  ld S = sin(alfa);
  ld C = cos(alfa);
  
  int i;

  if(pressed[SDLK_UP]    || pressed[SDLK_KP8]) REP(i,vert) roll(vpos[i][1], vpos[i][2], +S, C);
    
  if(pressed[SDLK_DOWN]  || pressed[SDLK_KP2]) REP(i,vert) roll(vpos[i][1], vpos[i][2], -S, C);
  
  if(pressed[SDLK_LEFT]  || pressed[SDLK_KP4]) REP(i,vert) roll(vpos[i][0], vpos[i][2], +S, C);
    
  if(pressed[SDLK_RIGHT] || pressed[SDLK_KP6]) REP(i,vert) roll(vpos[i][0], vpos[i][2], -S, C);

  if(pressed[SDLK_PAGEUP] || pressed[SDLK_KP9]) REP(i,vert) roll(vpos[i][0], vpos[i][1], -S, C);
  if(pressed[SDLK_PAGEDOWN] || pressed[SDLK_KP3]) REP(i,vert) roll(vpos[i][0], vpos[i][1], +S, C);

  if(pressed['z']) REP(i,vert) vpos[i] *= 1.03;
  if(pressed['x']) REP(i,vert) vpos[i] /= 1.03;  
  }

void frame();

void drawFrame() {
  if(!showit) { frame(); return; }

  SDL_LockSurface(surface);
  for(int y=0; y<SY; y++)
  for(int x=0; x<SX; x++) {
    putpix(x, y, 0); // ((x+y)*255) / (SX+SY));
    }
  
  frame();
  roll();

  SDL_UnlockSurface(surface);
  SDL_UpdateRect(surface, 0, 0, 0, 0);
  }

void applyKey(int e);

// do the graphic

void graphic(char *title) {
  if (SDL_Init(SDL_INIT_VIDEO) < 0) {
    printf("Failed to initialize SDL.\n");
    SDL_Quit();
    }

  surface = SDL_SetVideoMode(SX, SY, 32, 0);

  if(!surface) {
    printf("Failed to initialize SDL.\n");
    SDL_Quit();
    }

  SDL_WM_SetCaption(title, 0);
  
  drawFrame();

  while(!stop) {
    SDL_Event event;
    int e;
    bool ev = SDL_PollEvent(&event);
    if(ev) switch (event.type) {
      case SDL_KEYDOWN:
        e = event.key.keysym.sym;
        applyKey(e);
        if(e >=0 && e < 65536)
          pressed[e] = true;
        break;
        
      case SDL_KEYUP:
        e = event.key.keysym.sym;
        if(e >=0 && e < 65536)
        pressed[e] = false;
        break;
        
      case SDL_QUIT:
        stop = true;
        return;
      }
    else 
      drawFrame();
    }
  }

#undef main
